/*
 * Copyright 2000 Draagon Software, Inc. All Rights Reserved.
 *
 * This software is the proprietary information of Draagon Software, Inc.
 * Use is subject to license terms.
 *
 */

package com.draagon.wii;

import java.io.*;
import java.util.Enumeration;
import java.util.List;

import com.draagon.wii.util.NameValuePair;

/**
 * The WIIRequest interface defines the methods that any Draagon Web Integrator
 * Request objects must implement.  The WIIRequest object represents the request
 * from the client.
 *
 * @author  Doug Mealing
 * @version 1.1, 12/17/00
 * @since   JDK1.2
 */
public interface WIIRequest
{
  // NOTE: These are Draagon Web Integrator specific
  public String getWIIURL() throws WIIException;

  public String getSiteURL( WIISite site ) throws WIIException;
  public String getSiteURL() throws WIIException;

  public WIIKernel getKernel();
  public WIIContext getContext();
  public WIIDevice getDevice();
  public WIISite getSite();
  public WIIUser getUser();
  public WIITemplate getTemplate();
  public WIIConfiguration getConfiguration();

  public List<WIIRequestHeaders> getWIIRequestHeaders() throws WIIException;
  public List<WIIResponseHeaders> getWIIResponseHeaders() throws WIIException;
  public WIIConnector getWIIConnector() throws WIIException;
  public WIIStream getWIIStream( String content ) throws WIIException;
  public WIIService getWIIService( String name ) throws WIIException;

  public void setExternalInputStream( InputStream is );
  public InputStream getExternalInputStream();

  public void setExternalStatusCode( int code );
  public int getExternalStatusCode();

  public void setExternalContentType( String type );
  public String getExternalContentType();

  public List<NameValuePair> getRequestHeaders();

  // public Properties getWIIProperties();

  public void logError( String msg, Exception e );
  // public void displayAccessError( WIIResponse res ) throws IOException;

  // NOTE: These are from ServletRequest
  public String getCharacterEncoding();
  public int getContentLength();
  public String getContentType();
  // public ServletInputStream getInputStream();  // Use a regular stream, below
  public InputStream getInputStream() throws IOException;
  // public Locale getLocale();
  // public Enumeration getLocales();
  public String getParameter( String name );
  public Enumeration<String> getParameterNames();
  // public String[] getParameterValues( String name );
  public String getProtocol();
  public BufferedReader getReader() throws IOException;
  // public String getRealPath( String path );  // Deprecated. As of Version 2.1
  public String getRemoteAddr();
  public String getRemoteHost();
  // public String getScheme();
  public String getServerName();
  public int getServerPort();
  // public boolean isSecure();

  public Object getAttribute( String name );
  public void removeAttribute( String name );
  public void setAttribute( String name, Object o );
  // public Enumeration getAttributeNames();

  // NOTE: This is Java specific and should not be used!
  // public RequestDispatcher getRequestDispatcher( String path );

  // NOTE: These are from HTTPServletRequest
  // public String getAuthType();
  // public String getContextPath();
  // public Cookie[] getCookies();
  // public long getDateHeader( String name );

  // public String getHeader( String name );
  // public Enumeration getHeaderNames();

  // public Enumeration getHeaders( String name );
  // public int getIntHeader( String name );
  public String getMethod();
  public String getPathInfo();
  // public String getPathTranslated();
  public String getQueryString();
  // public String getRemoteUser();
  // public String getRequestedSessionId();
  // public String getRequestURI();
  // public String getServletPath();
  // NOTE: These are Java specific and should not be used!
  // public HttpSession getSession();
  // public HttpSession getSession( boolean create );
  // public Principal getUserPrincipal();
  // public boolean isRequestedSessionIdFromCookie();
  // public boolean isRequestedSessionIdFromUrl();
  // public boolean isRequestedSessionIdFromURL();
  // public boolean isRequestedSessionIdValid();
  // public boolean isUserInRole( String role );
}
