/**
 * Title:        Draagon Web Integrator<p>
 * Description:  Draagon Web Integrator is a web application integration framework.<p>
 * Copyright:    Copyright (c) Doug Mealing<p>
 * Company:      Draagon Software, Inc.<p>
 * @author Doug Mealing
 * @version 1.0
 */
package com.draagon.wii.config;

import com.draagon.wii.*;
import com.draagon.wii.internal.*;
import com.draagon.util.Configuration;

import java.util.ArrayList;
import java.util.List;
import java.util.Enumeration;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class PropertyConfiguration implements WIIConfiguration
{
  private Configuration mConfig = null;
  private static Log log = LogFactory.getLog( PropertyConfiguration.class );

  public void deleteProperty( String name )
  {
    mConfig.remove( name );
  }

  public void setProperty( String name, String val )
  {
    mConfig.setProperty( name, val );
  }

  public PropertyConfiguration( Configuration c )
  {
    mConfig = c;
  }

  public String getProperty( String name )
  {
    return mConfig.getProperty( name );
  }

  private List<String> parseHeaderClasses( String val )
  {
    List<String> tmp = new ArrayList<String>();
    if ( val == null ) return tmp;

    int i = 0;
    while( ( i = val.indexOf( "," )) >= 0 )
    {
      if ( i == 0 )
      {
        val = val.substring( 1 );
        continue;
      }

      String p = val.substring( 0, i ).trim();
      val = val.substring( i + 1 );

      tmp.add( p );
    }

    val = val.trim();
    if ( val.length() > 0 ) tmp.add( val );

    return tmp;
  }

  public List<String> getRequestHeaderClasses()
  {
    String val = mConfig.getProperty( "default.header.request.classes" );
    if ( val == null )
      log.warn( "(getRequestHeaderClasses) Could not find property: default.header.request.classes" );

    return parseHeaderClasses( val );
  }

/*  public void setRequestParamClasses( Vector v )
  {
  }

  public void setRequestParamClasses( WIISite site, Vector v )
  {
  }
*/
  public List<String> getRequestHeaderClasses( WIISite site )
  {
    String val = mConfig.getProperty( "site." + site.getName() + ".header.request.classes" );
    if ( val == null ) return getRequestHeaderClasses();

    return parseHeaderClasses( val );
  }

  public List<String> getResponseHeaderClasses()
  {
    String val = mConfig.getProperty( "default.header.response.classes" );
    if ( val == null )
      log.warn( "(getRequestHeaderClasses) Could not find property: default.header.response.classes" );

    return parseHeaderClasses( val );
  }

  public List<String> getResponseHeaderClasses( WIISite site )
  {
    String val = mConfig.getProperty( "site." + site.getName() + ".header.response.classes" );
    if ( val == null ) return getResponseHeaderClasses();

    return parseHeaderClasses( val );
  }

/*  public void setResponseParamClasses( Vector v ) {}
  public void setResponseParamClasses( WIISite site, Vector v ) {}
*/
  public String getConnectorClass()
  {
    String val = mConfig.getProperty( "default.connector.class" );
    if ( val == null )
      log.warn( "(getConnectorClass) Could not find property: default.connector.class" );

    return val;
  }

  public String getConnectorClass( WIISite site )
  {
    String classname = mConfig.getProperty( "site." + site.getName() + ".connector.class" );
    if ( classname == null ) return getConnectorClass();
    return classname;
  }

/*  public void setConnectorClass( String name ) {}
  public void setConnectorClass( WIISite site, String name ) {}
*/
  public String getStreamClass( String content )
  {
    String val = mConfig.getProperty( "default.stream.class" );
    if ( val == null )
      log.warn( "(getStreamClass) Could not find property: default.stream.class" );

    return val;
  }

  public String getStreamClass( WIISite site, String content )
  {
    String classname = mConfig.getProperty( "site." + site.getName() + ".stream.class" );
    if ( classname == null ) return getStreamClass( content );
    return classname;
  }
/*
  public void setStreamClass( String content, String name ) {}
  public void setStreamClass( WIISite site, String content, String name ) {}
*/
  public String getServiceClass( String service )
  {
    String val = mConfig.getProperty( "default.service." + service + ".class" );
    if ( val == null )
      log.warn( "(getServiceClass) Could not find property: default.service." +service+ ".class" );

    return val;
  }

  public String getServiceClass( WIISite site, String service )
  {
    String val = mConfig.getProperty( "site." + site.getName() + ".service." + service + ".class" );
    if ( val == null ) return getServiceClass( service );
    return val;
  }

/*  public void setServiceClass( String service, String name ) {}
  public void setServiceClass( WIISite site, String service, String name ) {}
*/
  public List<String> getAllSites()
  {
    List<String> tmp = new ArrayList<String>();

    for( Enumeration<?> e = mConfig.keys(); e.hasMoreElements(); )
    {
      String s = (String) e.nextElement();

      if ( s.indexOf( "site." ) == 0 && s.indexOf( ".url" ) > 0 )
      {
        s = s.substring( 5 );
        int i = s.indexOf( "." );
        if ( i < 0 ) continue;
        String site = s.substring( 0, i );
        if ( s.substring( i ).equals( ".url" )) tmp.add( site );
      }
    }

    return tmp;
  }

/*  public void deleteSite( String id )
  {
    for( Enumeration e = mConfig.keys(); e.hasMoreElements(); )
    {
      String s = (String) e.nextElement();

      if ( s.indexOf( "site." + id ) == 0 )
      {
        mConfig.remove( s );
      }
    }
  }

  public void addSite( String id, String url, String access )
  {
    mConfig.setProperty( "site." + id + ".url", url );
    mConfig.setProperty( "site." + id + ".access", access );
  }
*/
  public String getSiteProperty( WIISite site, String prop )
  {
    return mConfig.getProperty( "site." + site.getName() + "." + prop );
  }

  // public void setSiteProperty( WIISite site, String prop, String val ) {}

  public String getInternalServiceClass( String name )
  {
    String val = mConfig.getProperty( "internal." + name + ".class" );
    if ( val == null )
      log.warn( "(getInternalServiceClass) Could not find property: internal." +name+ ".class" );

    return val;
  }

  public String getInternalServiceProperty( InternalService is, String prop )
  {
    return mConfig.getProperty( "internal." + is.getName() + "." + prop );
  }

  /*public void setInternalServiceClass( String name, String val ) {}
  public void setInternalServiceProperty( InternalService is, String prop, String val ) {}
*/
  public String getServiceProperty( WIIService service, String prop )
  {
    String tmp = mConfig.getProperty( "default.service." + service.getName() + "." + prop );
    if ( tmp == null )
      log.warn( "(getServiceProperty) Could not find property: default.service." + service.getName() + "." + prop );

    return tmp;
  }

  public String getServiceProperty( WIISite site, WIIService service, String prop )
  {
    String tmp = mConfig.getProperty( site.getName() + ".service." + service.getName() + "." + prop );
    if ( tmp == null ) return getServiceProperty( service, prop );
    return tmp;
  }

/*  public void setServiceProperty( WIIService service, String prop, String val ) {}
  public void setServiceProperty( WIISite site, WIIService service, String prop, String val ) {}
*/

  public String getServiceTemplateProperty( WIITemplate template, WIIService service, String prop )
  {
    throw new Error( "(getServiceTemplateProperty) Method not supported!" );
  }

  public String getTemplateProperty( WIITemplate template, String prop )
  {
    throw new Error( "(getTemplateProperty) Method not supported!" );
  }
}
