/*
 * Copyright 2000 Draagon Software, Inc. All Rights Reserved.
 *
 * This software is the proprietary information of Draagon Software, Inc.
 * Use is subject to license terms.
 *
 */

package com.draagon.wii.headers;

import com.draagon.wii.*;
import com.draagon.wii.util.*;
import org.apache.commons.codec.binary.Base64;

import java.util.List;

/**
 * The RealmSSO request header object is currently very simplistic in that it
 * passes the same credentials used to authenticate against the portal to the
 * external system.  It passes them using HTTP Realm based authentication,
 * currently only in Basic mode.
 * <p>
 * A newer version of this object is needed so that it may allow the user to
 * enter their own set of credentials for the external system.
 *
 * @author  Doug Mealing
 * @version 1.1, 4/7/01
 * @see     com.draagon.wii.WIIRequestHeaders
 * @since   JDK1.2
 */
public class RealmSSO implements WIIRequestHeaders
{
  /**
   * Empty constructor
   */
  public RealmSSO()
  {
  }

  /**
   * Retrieves the users credentials from the WIIRequest and returns them in the
   * header list.
   *
   * @param req The WIIRequest object is used to retrieve the user's credentials
   * @param headers The headers containing the SSO data to be sent to the
   *                external system.
   */
  public void processRequestHeaders( WIIRequest req, List<NameValuePair> headers )
    throws WIIException
  {
      String [] creds = getCredentials( req );
      if ( creds == null ) return;

      // Retrieve the username and password
      String user = creds[ 0 ];
      String pass = creds[ 1 ];

      if ( user == null || pass == null ) return;

      String tmp = user + ":" + pass;
      tmp = "Basic " + Base64.encodeBase64String( tmp.getBytes() );

      // Add the header to the returned vector
      headers.add( new NameValuePair( "Authorization", tmp ));
  }

  protected String [] getCredentials( WIIRequest req )
    throws WIIException
  {
      String login = req.getConfiguration().getSiteProperty( req.getSite(), "realm.login" );
      String pass = req.getConfiguration().getSiteProperty( req.getSite(), "realm.pass" );

      if ( login != null && pass != null )
      {
        String [] tmp = new String[ 2 ];
        tmp[ 0 ] = login;
        tmp[ 1 ] = pass;
        return tmp;
      }

      WIIUser pu = req.getUser();
      if ( pu == null ) return null;

      return pu.getCredentials( req.getSite() );
  }
}
