/*
 * Copyright 2000 Draagon Software, Inc. All Rights Reserved.
 *
 * This software is the proprietary information of Draagon Software, Inc.
 * Use is subject to license terms.
 *
 */

package com.draagon.wii.services;

import com.draagon.wii.*;

/**
 * The BaseService is an abstract class used to derive new service classes that
 * will want to make use of common methods to store the WIIRequest object and
 * to retrive properties of the service/service.
 *
 * @author  Doug Mealing
 * @version 1.2, 12/17/00
 * @see     com.draagon.wii.WIIService
 * @since   JDK1.2
 */
public abstract class BaseService implements WIIService
{
  private String mName = null;
  protected WIIRequest mRequest = null;

  public String getName()
  {
    return mName;
  }

  public void setName( String name )
  {
    mName = name;
  }

  /**
   * Sets the WIIRequest object so that it can be used later.
   *
   * @param req The WIIRequest object to store.
   */
  public void setRequest( WIIRequest req )
  {
    mRequest = req;
  }

  /**
   * Empty constructor
   */
  public BaseService()
  {
  }

  /**
   * This method retrieves properties of the service requested by name.
   *
   * @param name The service property name to retrieve.
   * @return  The service property requested.
   */
  public String getProperty( String name )
  {
    return getProperty( name, null );
  }

  /**
   * This method retrieves properties of the service requested by name.
   *
   * @param name The service property name to retrieve.
   * @param def The default value to use if the property is not found in the
   *              configuration files.
   * @return  The service property requested.
   */
  public String getProperty( String name, String def )
  {
    String result = def;

    // Retrieve the WII Properties from the WIIRequest
    WIIConfiguration config = mRequest.getConfiguration();
    if ( config == null )
    {
      System.err.println( "(BaseService.getProperty) WIIConfiguration is null" );
      return def;
    }

    String tmp = (String) config.getServiceProperty( mRequest.getSite(), this, name );
    if ( tmp != null ) result = tmp;

    return result;
  }
}