/*
 * Copyright 2000 Draagon Software, Inc. All Rights Reserved.
 *
 * This software is the proprietary information of Draagon Software, Inc.
 * Use is subject to license terms.
 *
 */

package com.draagon.wii.services;

import com.draagon.wii.*;

import java.io.*;
import org.w3c.dom.*;

/**
 * The UserService currently outputs request user properties.  The external
 * application triggers this service by adding the following tag to the
 * returned content: <code>&lt;wii&gt;&lt;user property=&quot;WII_EMAIL&quot;
 *  /&gt;&lt;/wii&gt;</code>.
 * <p>
 * There are multiple valid property tags that can be used, but it is possible
 * to request properties specific to the portal integrated with.  It is
 * recommended that external applications use the Draagon Web Integrator (WII_) defined
 * properties for portability reasons.
 * <p>
 * The list of valid properties is defined below:
 * <ul>
 * <li>WII_USERNAME
 * <li>WII_PASSWORD
 * <li>WII_REALNAME
 * <li>WII_FIRSTNAME
 * <li>WII_LASTNAME
 * <li>WII_EMAIL
 * <li>WII_ADDRESS1
 * <li>WII_ADDRESS2
 * <li>WII_CITY
 * <li>WII_STATE
 * <li>WII_COUNTRY
 * <li>WII_ZIP
 * <li>WII_HOMEPHONE
 * <li>WII_COMPANY
 * <li>WII_TITLE
 * <li>WII_BUSPHONE
 * <li>WII_FAX
 * <li>WII_DEPARTMENT
 * <li>WII_PAGER
 * <li>WII_CELLPHONE
 * <li>WII_DEFAULTROLE
 * <li>WII_ROLES
 * </ul>
 *
 * @author  Doug Mealing
 * @version 1.2, 12/17/00
 * @see     com.draagon.wii.services.BaseService
 * @see     com.draagon.wii.WIIService
 * @since   JDK1.2
 */
public class UserService extends BaseService
{
  /**
   * Empty constructor
   */
  public UserService()
  {
  }

  /**
   * This method returns the requested user property string.
   *
   * @param node The XML DOM node to parse to retrieve the user property to
   *              return.
   * @exception  WIIException  This exception is not thrown when an error
   *                            occurs retrieving the user information or the
   *                            XML cannot be processed properly.
   * @return  The user property string to embed in the response.
   */
  public void process( Node node, WIIResponse res )
    throws IOException, WIIException
  {
    if ( node instanceof Element )
    {
      PrintWriter pw = res.getWriter();
      String val = ((Element) node ).getAttribute( "property" );
      // System.out.println( "User Property: " + val );
      pw.print( mRequest.getUser().get( val ));
    }
    else
      throw new WIIException( "Node was not an Element" );
  }
}
