/*
 * Copyright 2000 Draagon Software, Inc. All Rights Reserved.
 *
 * This software is the proprietary information of Draagon Software, Inc.
 * Use is subject to license terms.
 *
 */

package com.draagon.wii.services;

import java.io.*;
import com.draagon.wii.*;
import org.w3c.dom.Node;

/**
 * The WIIVersionService returns the version string for the WIIKernel when
 * requested by the external application.  The external application triggers
 * this service by adding the following tag to the returned content: <code>
 * &lt;wii&gt;&lt;version/&gt;&lt;/wii&gt;</code>.
 *
 * @author  Doug Mealing
 * @version 1.1, 12/17/00
 * @see     com.draagon.wii.WIIService
 * @since   JDK1.2
 */
public class WIIVersionService extends BaseService
{
  /**
   * Empty constructor
   */
  public WIIVersionService()
  {
  }

  /**
   * This method returns the version string of the WIIKernel.
   *
   * @param node The XML DOM node to parse, but it is not parsed in this
   *              service.
   * @exception  WIIException  This exception is not thrown by this service.
   * @return  The version string to embed in the response.
   */
  public void process( Node node, WIIResponse res )
    throws IOException, WIIException
  {
    PrintWriter pw = res.getWriter();
    pw.println( WIIKernel.VERSION );
  }
}
