/**
 * Title: Draagon Web Integrator<p> Description: Draagon Web Integrator is a web
 * application integration framework.<p> Copyright: Copyright (c) Doug
 * Mealing<p> Company: Draagon Software, Inc.<p>
 *
 * @author Doug Mealing
 * @version 1.0
 */
package com.draagon.wii.servlet;

import com.draagon.wii.*;
import com.draagon.wii.security.*;
import java.io.*;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class ServletSecurity implements SecuritySystem {

    private static Log log = LogFactory.getLog(ServletSecurity.class);

    public ServletSecurity() {
    }

    @Override
    public boolean checkAccess(Object security, WIIUser user) {
        String roles = (String) security;
        String user_roles = user.get(WIIUser.WII_ROLES);

        roles = roles.toLowerCase();
        user_roles = user_roles.toLowerCase();

        if (log.isDebugEnabled()) {
            log.debug("(checkAccess) user: [" + user_roles + "] access: [" + roles + "]");
        }

        if (roles.equals("")) {
            if (log.isDebugEnabled()) {
                log.debug("(checkAccess) No security defined, access granted");
            }
            return true;
        }

        if (roles.indexOf("user") >= 0 && user_roles.length() > 0) {
            if (log.isDebugEnabled()) {
                log.debug("(checkAccess) Authenticate user access defined and roles exist, access granted");
            }
            return true;
        }

        if (roles.indexOf("all") >= 0 || roles.indexOf("guest") >= 0) {
            if (log.isDebugEnabled()) {
                log.debug("(checkAccess) All access allowed, access granted");
            }
            return true;
        }

        try {
            StreamTokenizer st = new StreamTokenizer(new StringReader(user_roles));
            int type = 0;
            while ((type = st.nextToken()) != StreamTokenizer.TT_EOF) {
                if (type == StreamTokenizer.TT_WORD) {
                    String val = st.sval;
                    int i = roles.indexOf(val);
                    if (i >= 0) {
                        // Check if it starts with a ' ' or a ','
                        if (i != 0) {
                            int c = roles.charAt(i - 1);
                            if (c != ' ' && c != ',' && c != '\n' && c != '\t' && c != '\r') {
                                continue;
                            }
                        }

                        // Check to see if it ends with a ' ' or a ','
                        if (roles.length() > i + val.length()) {
                            int c = roles.charAt(i + val.length());
                            if (c != ' ' && c != ',' && c != '\n' && c != '\t' && c != '\r') {
                                continue;
                            }
                        }

                        // Must be the value we are looking for
                        if (log.isDebugEnabled()) {
                            log.debug("(checkAccess) Roles match, access granted");
                        }

                        return true;
                    }
                }
            }
        } catch (Exception e) {
            log.error("(ServletSecurity) Error parsing - user: [" + user_roles + "] access: [" + roles + "]", e);
            return false;
        }

        if (log.isDebugEnabled()) {
            log.debug("(checkAccess) Not matching roles, access denied");
        }

        return false;
    }
}
