/**
 * Title: Draagon Web Integrator<p> Description: Draagon Web Integrator is a web
 * application integration framework.<p> Copyright: Copyright (c) Doug
 * Mealing<p> Company: Draagon Software, Inc.<p>
 *
 * @author Doug Mealing
 * @version 1.0
 */
package com.draagon.wii.servlet;

import com.draagon.wii.*;
import javax.servlet.http.*;

public abstract class ServletUser implements WIIUser {

    private HttpSession mSession = null;

    // An array of this inner class is used to hold the individual properties for
    //  each user.
    public final class UserDetail {

        public final String key;
        public volatile String data = "";

        public UserDetail(String key) {
            this.key = key;
        }
    }
    
    public final UserDetail[] mDataArray;

    protected ServletUser() {
        this(null);
    }

    public ServletUser(HttpSession session) {
        mSession = session;
        mDataArray = new UserDetail[22];

        // These key values can be used to lookup a property by the key string name
        mDataArray[ 0] = new UserDetail("WII_USERNAME");
        mDataArray[ 1] = new UserDetail("WII_PASSWORD");
        mDataArray[ 2] = new UserDetail("WII_REALNAME");
        mDataArray[ 3] = new UserDetail("WII_FIRSTNAME");
        mDataArray[ 4] = new UserDetail("WII_LASTNAME");
        mDataArray[ 5] = new UserDetail("WII_EMAIL");
        mDataArray[ 6] = new UserDetail("WII_ADDRESS1");
        mDataArray[ 7] = new UserDetail("WII_ADDRESS2");
        mDataArray[ 8] = new UserDetail("WII_CITY");
        mDataArray[ 9] = new UserDetail("WII_STATE");
        mDataArray[ 10] = new UserDetail("WII_COUNTRY");
        mDataArray[ 11] = new UserDetail("WII_ZIP");
        mDataArray[ 12] = new UserDetail("WII_HOMEPHONE");
        mDataArray[ 13] = new UserDetail("WII_COMPANY");
        mDataArray[ 14] = new UserDetail("WII_TITLE");
        mDataArray[ 15] = new UserDetail("WII_BUSPHONE");
        mDataArray[ 16] = new UserDetail("WII_FAX");
        mDataArray[ 17] = new UserDetail("WII_DEPARTMENT");
        mDataArray[ 18] = new UserDetail("WII_PAGER");
        mDataArray[ 19] = new UserDetail("WII_CELLPHONE");
        mDataArray[ 20] = new UserDetail("WII_DEFAULTROLE");
        mDataArray[ 21] = new UserDetail("WII_ROLES");
    }
    /* End of constructor */

    protected abstract String getInternal(String name);

    /**
     * Returns a user property for the passed key value. If the method can not
     * locate the user property in the list of standard properties, it will
     * allow the portal specific user object to perform the lookup.
     *
     * @param String User property key name
     *
     * @return <code>String</code> User property value
     */
    public String get(String name) {
        if (name.indexOf("WII_") == 0) {
            // Lookup the WII_ name in the data array and return the data value
            for (int i = 0; i < mDataArray.length; i++) {
                if (mDataArray[ i].key.equals(name)) {
                    return mDataArray[ i].data;
                }
            }
        }

        // Must be an internal portal property
        return getInternal(name);
    }
    /* End of get( String ) method */

    /**
     * Returns a user property for the passed key constant.
     *
     * @param int User property key constant
     *
     * @return <code>String</code> User property value
     */
    @Override
    public String get(int index) {
        // Make sure the index is valid
        if (index >= mDataArray.length) {
            return null;
        }
        return mDataArray[ index].data;
    }

    @Override
    public Object getAttribute(String name) {
        if (mSession == null) {
            return null;
        }

        return mSession.getAttribute("wii.ext." + name);
    }

    @Override
    public void setAttribute(String name, Object value) {
        if (mSession == null) {
            return;
        }

        mSession.setAttribute("wii.ext." + name, value);
    }

    @Override
    public void removeAttribute(String name) {
        if (mSession == null) {
            return;
        }

        mSession.removeAttribute("wii.ext." + name);
    }

    @Override
    public String[] getCredentials(WIISite site) {
        String[] creds = new String[4];

        creds[ 0] = get(WII_USERNAME);
        creds[ 1] = get(WII_PASSWORD);
        creds[ 2] = get(WII_DEFAULTROLE);
        creds[ 3] = get(WII_ROLES);

        return creds;
    }

    /* End of get( int ) method */
}
