/*
 * Copyright 2000 Draagon Software, Inc. All Rights Reserved.
 *
 * This software is the proprietary information of Draagon Software, Inc.
 * Use is subject to license terms.
 *
 */

package com.draagon.wii.servlet;

import java.io.*;
import javax.servlet.http.*;

import com.draagon.wii.*;

public abstract class WIIServletResponseAdapter implements WIIResponse
{
  protected HttpServletResponse mRes = null;
  protected boolean isStream = false;
  protected boolean isWriter = false;

  public WIIServletResponseAdapter( HttpServletResponse res )
  {
    mRes = res;
  }

  // NOTE: These are OS/Web specific
  public HttpServletResponse getServletResponse()
  {
    return mRes;
  }

  public OutputStream getOutputStream()
    throws IOException
  {
	if ( isWriter ) {
		mRes.getWriter().flush();
		isWriter = false;
	}
	isStream = true;
    return mRes.getOutputStream();
  }

  public PrintWriter getWriter()
    throws IOException
  {
	if ( isStream ) {
		mRes.getOutputStream().flush();
		isStream = false;
	}
	isWriter = true;
    return mRes.getWriter();
  }

  
  /** Flush the output stream or writer */
  public void flush() throws IOException 
  {
	  if ( !isStream && !isWriter ) return;
	  if ( isStream ) 
	  {
		  mRes.getOutputStream().flush();
	  }
	  else
	  {
		  mRes.getWriter().flush();
	  }
  }

  /** Flush and close the output stream or writer */
  public void close() throws IOException 
  {
	  if ( !isStream && !isWriter ) return;
	  if ( isStream ) 
	  {
		  mRes.getOutputStream().flush();
		  mRes.getOutputStream().close();
	  }
	  else
	  {
		  mRes.getWriter().flush();
		  mRes.getWriter().close();
	  }
  }


  public void setContentType( String type )
  {
    mRes.setContentType( type );
  }

  public void addHeader( String name, String value )
  {
    // Adding cookies doesn't work with setHeader, so do it manually
    if ( name.compareToIgnoreCase( "set-cookie" ) == 0 )
    {
      int j = value.indexOf( '=' );
      if ( j > 0 )
      {
        String n = value.substring( 0, j );
        String v = value.substring( j + 1 );

        // System.out.println( "Setting cookie: [" + n + "][" + v + "]" );

        mRes.addCookie( new Cookie( n, v ));
      }
    }
    else
      mRes.setHeader( name, value );
  }

  public void setStatusCode( int sc )
  {
    mRes.setStatus( sc );
  }

  // NOTE: These are from ServletRequest
/*  public void flushBuffer()
    throws IOException
  {
    mRes.flushBuffer();
  }

  public int getBufferSize()
  {
    return mRes.getBufferSize();
  }

  public String getCharacterEncoding()
  {
    return mRes.getCharacterEncoding();
  }

  public Locale getLocale()
  {
    return mRes.getLocale();
  }

  public boolean isCommitted()
  {
    return mRes.isCommitted();
  }

  public void reset()
  {
    // This is not implemented in JSDK 1.2
    //mRes.reset();
  }

  public void setBufferSize( int size )
  {
    mRes.setBufferSize( size );
  }

  public void setContentLength( int len )
  {
    mRes.setContentLength( len );
  }

  public void setLocale( Locale loc )
  {
    mRes.setLocale( loc );
  }

  public void addCookie( Cookie cookie )
  {
    mRes.addCookie( cookie );
  }

  public void addDateHeader( String name, long date )
  {
    mRes.addDateHeader( name, date );
  }

  public void addIntHeader( String name, int value )
  {
    mRes.addIntHeader( name, value );
  }

  public boolean containsHeader( String name )
  {
    return mRes.containsHeader( name );
  }

  public String encodeRedirectURL( String url )
  {
    return mRes.encodeRedirectURL( url );
  }

  public String encodeURL( String url )
  {
    return mRes.encodeURL( url );
  }

  public void sendError( int sc ) throws IOException
  {
    mRes.sendError( sc );
  }

  public void sendError( int sc, String msg ) throws IOException
  {
    mRes.sendError( sc, msg );
  }

  public void sendRedirect( String location ) throws IOException
  {
    mRes.sendRedirect( location );
  }

  public void setDateHeader( String name, long date )
  {
    mRes.setDateHeader( name, date );
  }

  public void setHeader( String name, String value )
  {
    mRes.setHeader( name, value );
  }

  public void setIntHeader( String name, int value )
  {
    mRes.setIntHeader( name, value );
  }
*/
}
